;	
;	Some minor changes to align crucial routines with standard MIKBUG addresses, and
;	to match MIKBUG variables at $A000.   Since we have 8k available, the 512 bytes
;	that would normally be occupied by MIKBUG ($E000-E1FF) contains only initialisation
;	and MIKBUG-compatible vectors to routines from $E200 onwards. Added some commands:
;	'P'=punch S19 records from memory, U=TSC Microbasic (Robert Uiterwyk) (MON to return)
;	'+' is hex arithmetic,takes 2 args & gives sum, difference & relative jump offset
;	Changed to brief prompt with 'H' is help, rather than full command list every time.
;	Init bug fixed, ACIA was being reset whilst transmitting last character of menu. 
;	RTI dissassembly bug fixed at label NOTRTS, value $38 should be $3B
;	To add: allow overlapped block moves, 16-bit find. Changes by Phil_G  08/02/2021 23:34
; 
        titl "MIKES-M6802 OPERATING SYSTEM VERSION 11"
;	REV 012   (13?)
;
;	A	SET A REGISTER
;	B	SET B REGISTER
;	X	SET X REGISTER
;	S	SET STACK POINTER
;	C	SET PROGRAM COUNTER
;
;	M	MOVE MEMORY
;	E	CHANGE MEMORY
;	G	GO TO PROGRAM
;	R	PRINT
;	T	TRACE PROGRAM
;	@	ASCII CONVERSION
;	H	PRINTER ON
;	V	VIEW MEMORY
;	I	FILL MEMORY
;	J	JUMP TO TARGET PROGRAM
;	F	FIND 
;	Q	HARDWARE LOCATION
;	D	DISASSEMBLE CODE
;	K	CONTINUE AFTER BREAK
;	1	BREAKPOINT ONE
;	2	BREAKPOINT TWO
;	&	S1 LOAD PROGRAMME
;	O	ECHO ON
;	N	ECHO OFF	
;
ACIACS		EQU	$8004
ACIADA		EQU	$8005
;
PIAD1A		EQU	$8800
PIAS1A		EQU	$8801
PIAD1B		EQU	$8802
PIAS1B		EQU	$8803
;
;		CRITICAL MIKBUG ENTRY POINTS
;
;LOAD19		EQU	$E040 
;BADDR 		EQU	$E047
;BYTE 		EQU	$E055 
;OUTCH 		EQU	$E075 
;INCH 		EQU	$E078 	
;PDATA2		EQU	$E07B 
;PDATA1		EQU	$E07E 
;OUT2H 		EQU	$E0BF 
;OUT4HS		EQU	$E0C8 
;OUT2HS		EQU	$E0CA 
;START		EQU	$E0D0 
;CONTRL		EQU	$E0E3 
;MCLOFF		EQU	$E19C 
;MCL		EQU	$E19D 
;INEEE		EQU	$E1AC 
;OUTEEE		EQU	$E1D1 
;
;	OPT	MEMORY
;
;	SCRATCH AREA
;
	ORG $A000
;
IOV		RMB 2
BEGA 		RMB 2
ENDA 		RMB 2
NIO 		RMB 2
SP 		RMB 2
CSUM		RMB 1
BYTECT		RMB 1
XHI 		RMB 1
XLOW 		RMB 1
TEMP 		RMB 1
TW		RMB 2
MCONT		RMB 1
XTEMP		RMB 2
SWIPTR		RMB 46
STACK		RMB 32
TSTACK		RMB 2
TFLAG		RMB 1
BKFLG		RMB 1
PRINTR		RMB 3
BFLAG 		RMB 1
MFLAG		RMB 1
XFLAG		RMB 1
BITE2		RMB 1
BITE3		RMB 15
OPSAVE		RMB 1
PB1		RMB 1
PB2		RMB 1
PB3		RMB 1
PC1		RMB 1
PC2		RMB 1
BPOINT		RMB 3
BKFLG2		RMB 1
TEMPX1		RMB 2	; S-Load Temporary X register
ADDRESS		RMB 1	; Build 16 bit address byte 1
ADDRESS1	RMB 1	; Build 16 bit address byte 2
BYTESTORE	RMB 1	; Save Byte Count
TEMPA		RMB 1	; Save A Register
BUFFER		RMB 2
ACIAT 		RMB 1
ECHO 		RMB 1
COPYFR		RMB 2
COPYTO		RMB 2
LENGTH		RMB 1
LENGTH2		RMB 1
;
	ORG	$E000
;
;	ENTER POWER ON SEQUENCE
;
START	EQU *
	LDS #STACK
	STS SP
	CLR ECHO
	LDX #SFE
	STX SWIPTR
	STX NIO
;
;	ACIA INITIALISE
;
	LDAA	#$03	; RESET CODE
	STAA	ACIACS
	NOP
	NOP
	NOP
	LDAA	#$15	; 8N1 NON-INTERRUPT
	STA A 	ACIACS
	STA A	ACIAT
	JMP	CONTRL
;
	ORG $E040 
	JMP LOAD19
	ORG $E047
	JMP BADDR
	ORG $E055
	JMP BYTE
	ORG $E075
	JMP OUTCH
	ORG $E078
	JMP INCH
	ORG $E07B
	JMP PDATA2
	ORG $E07E
	JMP PDATA1
	ORG $E0BF
	JMP OUT2H
	ORG $E0C8
	JMP OUT4HS
	ORG $E0CA
	JMP OUT2HS
	ORG $E0D0
	JMP START
	ORG $E0E3
	JMP CONTRL
	ORG $E19C
	FCB $0D       ; READER OFF
	ORG $E19D
	FCB $0D,$0A,4   ; C/R,L/F,PUNCH
	ORG $E1AC
	JMP INEEE
	ORG $E1D1
	JMP OUTEEE
;
	ORG	$E200
;	COMMAND CONTROL
;
CONTRL	LDS #STACK	; relic?		
;	LDS #TSTACK	; SET CONTRL STACK POINTER
	CLR TFLAG
	CLR BKFLG
	CLR BKFLG2
	LDX #MONPR
	BSR PDATA1
	BSR INCH
	TAB
	JSR OUTS	; print ascii space after command letter
;
; CHECK IF COMMAND IS VALID AND JUMP TO APPLICATION
;
	LDX #FUTABL
NXTCHR	CMP B 0,X
	BEQ GOODCH
	INX
	INX
	INX
	CPX #TBLEND
	BNE NXTCHR
	JMP CONTRL
GOODCH	LDX 1,X
	JMP 0,X
;
;  IRQ INTERUPT SEQUENCE
;
IO	LDX IOV
	JMP 0,X
;
;  NMI SEQUENCE
;
POWDWN	LDX NIO
	JMP 0,X
;
;  SWI SEQUENCE
;
SWI	LDX SWIPTR
	JMP 0,X
LOAD19	LDA A #$3F
	BSR OUTCH
C1	BRA CONTRL
;
;  BUILD ADDRESS
;
BADDR	BSR BYTE
	STA A XHI
	BSR BYTE
	STA A XLOW
	LDX XHI
	RTS
;
;  INPUT ONE BYTE
;
BYTE	BSR INHEX
	ASL A
	ASL A
	ASL A
	ASL A
	TAB
	BSR INHEX
	ABA
	RTS
;
;  OUTPUT LEFT HEX NUMBER
;
OUTHL	LSR A
	LSR A
	LSR A
	LSR A
;
;  OUTPUT RIGHT HEX NUMBER
;
OUTHR	AND A #$F
	ADD A #$30
	CMP A #$39
	BLS OUTCH
	ADD A #$7
OUTCH	JMP OUTEEE
INCH	JMP INEEE
;
;  PRINT MSG TERMINATED BY $04
;
PDATA2	BSR OUTCH
	INX
PDATA1	LDA A 0,X
	CMP A #$4
	BNE PDATA2
	RTS
;
; CHANGE MEMORY
;
CHANGE	BSR BADDR
CHA51	LDX #MCL
	BSR PDATA1
	BSR OUTXHI
	BSR OUT2HS
	STX XHI
	BSR INCH
	CMP A #$20
	BEQ CHA51
	CMP A #$5E
	BNE CHM1
	DEX
	DEX
	STX XHI
	BRA CHA51
CHM1	BSR INHEX+2
	BSR BYTE+2
	DEX
	STA A 0,X
	CMP A 0,X
	BEQ CHA51
;
XBK	BRA LOAD19
;
INHEX	BSR INCH
	SUB A #$30
	BMI C1
	CMP A #$9
	BLE IN1HG
	CMP A #$11
	BMI C1
	CMP A #$16
	BGT C1
	SUB A #$7
IN1HG	RTS
;
;
OUT2H	LDA A 0,X
	BSR OUTHL
	LDA A 0,X
	INX
	BRA OUTHR
;
OUT4HS	BSR OUT2H
OUT2HS	BSR OUT2H
OUTS	LDA A #$20
	BRA OUTCH
;
; SET BREAK POINTS
;
BKPNT2	JSR ADDR
	STX PC1
	LDA A 0,X
	STA A BKFLG2
	BEQ XBK
	LDA A #$3F
	STA A 0,X
BKPNT	JSR ADDR
	STX PB2
	LDA A 0,X
	STA A BKFLG
	BEQ XBK
	LDA A #$3F
	STA A 0,X
	JSR CRLF
;
; FALL INTO GO COMMAND
;
CONTG	LDS SP
	RTI
;
; PRINT XHI ADDRESS SUB
;
OUTXHI	LDX #XHI
	BSR OUT4HS
	LDX XHI
	RTS
;
; VECTORED SWI ROUTINE
;
SFE	STS SP
	TSX
	TST 6,X
	BNE *+4
	DEC 5,X
	DEC 6,X
	LDS #TSTACK
	TST TFLAG
	BEQ PRINT
	LDX PC1
	LDA A OPSAVE
	STA A 0,X
	TST BFLAG
	BEQ DISPLY
	LDX BPOINT
	LDA A BPOINT+2
	STA A 0,X
DISPLY	JMP RETURN
;
; PRINT REGISTERS
;
PRINT	LDX SP
	LDA A #6
	STA A MCONT
	LDA B 1,X
	ASL B
	ASL B
	LDX #CSET
;
DSOOP	LDA A #$2D
	ASL B
	BCC DSOOP1
	LDA A 0,X
DSOOP1	JSR OUTEEE
	INX
	DEC MCONT
	BNE DSOOP
	LDX #BREG
	BSR PDAT
	LDX SP
	INX
	INX
	JSR OUT2HS
	STX TEMP
	LDX #AREG
	BSR PDAT
	LDX TEMP
	JSR OUT2HS
	STX TEMP
	LDX #XREG
	BSR PDAT
	LDX TEMP
	BSR PRTS
	STX TEMP
	TST TFLAG
	BNE PNTS
	LDX #PCTR
	BSR PDAT
	LDX TEMP
	BSR PRTS
PNTS	LDX #SREG
	BSR PDAT
	LDX #SP
	TST TFLAG
	BNE PRINTS
	BSR PRTS
;
; CHECK IF ANY BREAK POINTS ARE SET
;
	LDA A BKFLG
	BNE C2
	LDX PB2
	STA A 0,X
	LDA A BKFLG2
	BEQ C2
	LDX PC1
	STA A 0,X
C2	BRA CR8
PDAT	JMP PDATA1
;
; SET ECHO FUNCTION
;
ECHON	CLR B
PRNTON	NEG B
ECHOFF	STA B ECHO
	BRA CR8
;
;  PRINT STACK POINTER
;
PRINTS	LDA B 0,X
	LDA A 1,X
	ADD A #7
	ADC B #0
	STA B TEMP
	STA A TEMP+1
	LDX #TEMP
PRTS	JMP OUT4HS
;
CR8	BRA IFILL1
;
;     SAVE X REGISTER
;
SAV	STX XTEMP
	RTS
;
;	INPUT ONE CHAR INTO A-REGISTER
;
INEEE 	BSR	SAV
IN1	LDAA	ACIACS
	ASRA
	BCC	IN1		;RECEIVE NOT READY
	LDAA	ACIADA		;INPUT CHARACTER
	ANDA	#$7F		;RESET PARITY BIT
	CMPA	#$7F
	BEQ	IN1		;IF RUBOUT, GET NEXT CHAR
	TST ECHO
	BLE OUTEEE
	RTS
;
;	OUTPUT ONE CHAR 
;
OUTEEE	PSH	A
OUTEEE1 LDA A	ACIACS
	ASR A
	ASR A
	BCC	OUTEEE1
	PUL A
	STA A	ACIADA
	RTS
;
;  HERE ON JUMP COMMAND
;
JUMP	LDX #TOADD
	BSR ENDADD+3
	LDS #STACK
	JMP 0,X
;
;  ASCII IN "@" COMMAND
;
ASCII	BSR BAD2
	INX
ASC01	DEX
ASC02	BSR INEEE
	CMP A #$8
	BEQ ASC01
	STA A 0,X
	CMP A #$4
	BEQ CR9
	INX
	BRA ASC02
;
;  FILL MEMORY "I" COMMAND
;
IFILL	BSR LIMITS
	BSR VALUE
	LDX BEGA
	DEX
IFILL2	INX
	STA A 0,X
	CPX ENDA
	BNE IFILL2
IFILL1	BRA CR9
;
;  INPUT DATA SUB ROUTINE
;
BAD2	LDX #FROMAD
	BRA *+5
ENDADD	LDX #THRUAD
	JSR PDATA1
	JMP BADDR
LIMITS	BSR BAD2
	STX BEGA
	BSR ENDADD
	STX ENDA
	JMP CRLF
ADDR	LDX ADASC
	BRA ENDADD+3
VALUE	LDX #VALASC
	JSR PDATA1
	JMP BYTE
;
; BLOCK MOVE "M" COMMAND   Phil_G: doesnt account for overlapping
;
MOVE	BSR LIMITS	; GET BEGA & ENDA
	LDX #TOADD
	BSR ENDADD+3	; ENDADD+3 PRINTS 'WHERE TO' & GETS ADDRESS IN XHI & XLO
	BSR MOVE2
CR9	JMP CONTRL
;
MOVE2	LDX BEGA
	DEX
BMC1	INX
	LDA A 0,X
	STX BEGA
	LDX XHI
	STA A 0,X
	INX
	STX XHI
	LDX BEGA
	CPX ENDA
	BNE BMC1
	RTS
;
;  SEARCH MEMORY "S" COMMAND
;
FIND	BSR LIMITS
	BSR VALUE
	TAB
	LDX BEGA
	DEX
SMC1	INX
	LDA A 0,X
	CBA
	BNE SMC2
	STX XHI
	BSR CRLF
	JSR OUTXHI
SMC2	CPX ENDA
	BNE SMC1
	BRA CR9
;
;  SUB ROUTINE TO ADD SPACE
;
SKIP	LDA A #$20
	JSR OUTEEE
	DEC B 
	BNE SKIP
	RTS
;
;  PRINT BYTE IN A REGISTER
;
PNTBYT	STA A BYTECT
	LDX #BYTECT
	JMP OUT2H
;
;  CARRIAGE RETURN NON PROMPT
;
CRLF	LDX #CRLFAS
	JMP PDATA1
;
;  DISASSEMBLE "D" COMMAND
;
DISSA	JSR BAD2
	BRA DISS
;
;  TRACE COMMAND "T"
;
TRACE	JSR 	BAD2
	BSR 	CRLF
	LDX 	SP
	LDA B 	XHI
	STA B 	6,X
	LDA A 	XLOW
	STA A 	7,X
KONTIN	INC 	TFLAG
RETURN	JSR 	PRINT
	LDX 	SP
	LDX 	6,X
DISS	STX 	PC1
DISIN	BSR 	CRLF
	LDX 	#PC1
	JSR 	OUT4HS
	LDX 	#BFLAG
	LDA A 	#5
CLEAR	CLR 	0,X
	INX
	DEC A
	BNE 	CLEAR
	LDX 	PC1
	LDA B 0,X
	JSR 	OUT2HS
	STX 	PC1
	LDA A 	0,X
	STA A 	PB2
	LDA A 	1,X
	STA A 	PB3
	STA B 	PB1
	TBA
	JSR 	TBLKUP
	LDA A 	TEMP
	CMP A 	#$2A
	BNE 	OKOP
	JMP 	NOTBB
OKOP	LDA A PB1
	CMP A #$8D
	BNE 	NEXT
	INC 	BFLAG
	BRA 	PUT1
NEXT	AND A #$F0
	CMP A #$60
	BEQ 	ISX
	CMP A #$A0
	BEQ  	ISX
	CMP A	#$E0
 	BEQ  	ISX
	CMP A	#$80
	BEQ 	IMM
	CMP A #$C0
	BNE 	PUT1
IMM	INC 	MFLAG
	LDX 	#SPLBD0
	BRA 	PUT
ISX	INC 	XFLAG
	LDA A PB2
	JSR	PNTBYT
	LDX 	#COMMX
PUT	JSR 	PDATA1
PUT1	LDX 	PC1
	LDA A PB1
	CMP A #$8C
	BEQ 	BYT3
	CMP A #$8E
	BEQ 	BYT3
	CMP A #$CE  
	BEQ 	BYT3
	AND A #$F0
	CMP A #$20
	BNE 	NOTB
	INC 	BFLAG
	BRA 	BYT2
NOTB	CMP A #$60
	BCS 	BYT1
	AND A #$30
	CMP A #$30
	BNE 	BYT2
BYT3	INC 	BITE3
	TST 	MFLAG
	BNE 	BYT31
	LDA A #$24
	JSR 	OUTEEE
BYT31	LDA A 0,X
	INX
	STX 	PC1
	JSR 	PNTBYT
	LDX 	PC1
	BRA 	BYT21
BYT2	INC 	BITE2
BYT21	LDA A 0,X
	INX
	STX 	PC1
	TST 	XFLAG
	BNE 	BYT1
	TST 	BITE3
	BNE 	BYT22
	TST 	MFLAG
	BNE 	BYT22
	TAB
	LDA A #$24
	JSR 	OUTEEE
	TBA
BYT22	JSR 	PNTBYT
BYT1	TST 	BFLAG
	BEQ 	NOTBB
	LDA B #3
	JSR 	SKIP
	CLR A
	LDA B PB2
	BGE 	DPOS
	LDA A #$FF
DPOS	ADD B PC2
	ADC A PC1
	STA A BPOINT
	STA B BPOINT+1
	LDX 	#BPOINT
	JSR 	OUT4HS
;
; PRINT ASCII VALUE OF INST
;
NOTBB	LDA B #$D
	LDA A #1
	TST BITE2
	BEQ PAVOI3
	LDA B #1
	TST BFLAG
	BNE PAVOI2
	LDA B #8
	TST MFLAG
	BNE PAVOI2
	TST MFLAG
	BNE PAVOI2
	LDA B #9
PAVOI2	LDA A #2
	BRA PAVOI8
;
PAVOI3	TST BITE3
	BEQ PAVOI8
	LDA A #3
	LDA B #6
	TST MFLAG
	BEQ PAVOI8
	LDA B #5
PAVOI8	PSH A
	JSR SKIP
	PUL B
	LDX #PB1
PAVOI4	LDA A 0,X
	CMP A #$20
	BLE PAVOI5
	CMP A #$60
	BLE PAVOI9
PAVOI5	LDA A #$2E
PAVOI9	INX
	JSR OUTEEE
	DEC B
	BNE PAVOI4
NOT1	JSR INEEE
	TAB
	JSR OUTS
	CMP B #$20
	BEQ DOT
;
;  CHECK INPUT COMMAND
;  A, B, C, X, OR S
;
CKCBA	LDX SP
	INX
	CMP B #$43
	BEQ RDC
	INX
	CMP B #$42
	BEQ RDC
	INX
	CMP B #$41
	BEQ RDC
	INX
	CMP B #$58
	BEQ RDX
	LDX #SP
	CMP B #$53
	BNE RETNOT
RDX	JSR BYTE
	STA A 0,X
	INX
RDC	JSR BYTE
	STA A 0,X
	JSR CRLF
	JSR PRINT
;
;  WILL RETURN HERE IN TRACE
;
	BRA NOT1
RETNOT	JMP CONTRL
DOT	TST TFLAG
	BNE DOT1
	JMP DISIN
;
DOT1	LDA B #$3F
	LDA A PB1
	CMP A #$8D
	BNE TSTB
	LDX BPOINT
	STX PC1
	CLR BFLAG
TSTB	TST BFLAG
	BEQ TSTJ
	LDX BPOINT
	LDA A 0,X
	STA A BPOINT+2
	STA B 0,X
	BRA EXEC
;
TSTJ	CMP A #$6E
	BEQ ISXD
	CMP A #$AD
	BEQ ISXD
	CMP A #$7E
	BEQ ISJ
	CMP A #$BD
	BNE NOTJ
ISJ	LDX PB2
	STX PC1
	BRA EXEC
ISXD	LDX SP
	LDA A 5,X
	ADD A PB2
	STA A PC2
	LDA A 4,X
	ADC A #0
	STA A PC1
	BRA EXEC
;
NOTJ	LDX SP
	CMP A #$39
	BNE NOTRTS
NOTJ1	LDX 8,X
	BRA EXR
;
NOTRTS	CMP A #$3B
	BNE NOTRTI
	LDX 13,X
EXR	STX PC1
NOTRTI	CMP A #$3F
	BEQ NONO
	CMP A #$3E
	BEQ NONO
;
EXEC	LDX PC1
	LDA A 0,X
	STA A OPSAVE
	STA B 0,X
	CMP B 0,X
	BNE CKROM
	JMP CONTG
;
NONO	JMP LOAD19
;
CKROM	LDA A PC1
	CMP A #$E0
	BCS NONO
;
;  GET JSR OR JMP
;
	LDX SP
	LDA A PB1
	CMP A #$7E
	BEQ NOTJ1
	CMP A #$BD
	BNE NONO
	LDX 6,X
	INX
	INX
	INX
	BRA ISJ+3
;
;  OP CODES LOOKUP 
;
TBLKUP	CMP A #$40
	BCC IMLR6
IMLR1	JSR PNT3C
	LDA A PB1
	CMP A #$32
	BEQ IMLR3
	CMP A #$36
	BEQ IMLR3
	CMP A #$33
	BEQ IMLR4
	CMP A #$37
	BEQ IMLR4
IMLR2	LDX #BLANK
	BRA IMLR5
;
IMLR3	LDX #PNTA
	BRA IMLR5
;
IMLR4	LDX #PNTB
IMLR5	JMP PDATA1
IMLR6	CMP A #$4E
	BEQ IMLR7
	CMP A #$5E
	BNE IMLR8
;
IMLR7	CLR A
	BRA IMLR1
;
IMLR8	CMP A #$80
	BCC IMLR9
	AND A #$4F
	JSR PNT3C
	LDA A TEMP
	CMP A #$2A
	BEQ IMLR2
	LDA A PB1
	CMP A #$60
	BCC IMLR2
	AND A #$10
	BEQ IMLR3
	BRA IMLR4
;
IMLR9	AND A #$3F
	CMP A #$F
	BEQ IMLR7
	CMP A #$7
	BEQ IMLR7
	AND A #$F
	CMP A #$3
	BEQ IMLR7
	CMP A #$C
	BGE IMLR10
	ADD A #$50
	JSR PNT3C
	LDA A PB1
	AND A #$40
	BEQ IMLR3
	BRA IMLR4
;
IMLR10	LDA A PB1
	CMP A #$8D
	BNE IMLR11
	LDA A #$53
	BRA IMLR1
;
IMLR11	CMP A #$C0
	BCC IMLR12
	CMP A #$9D
	BEQ IMLR7
	AND A #$F
	ADD A #$50
	BRA IMLR13
;
IMLR12	AND A #$F
	ADD A #$52
	CMP A #$60
	BLT IMLR7
;
IMLR13	JMP IMLR1
;
PNT3C	CLR B
	STA A TEMP
	ASL A
	ADD A TEMP
	ADC B #$0
	LDX #TBL
	STX XTEMP
	ADD A XTEMP+1
	ADC B XTEMP
	STA B XTEMP
	STA A XTEMP+1
	LDX XTEMP
	LDA A 0,X
	STA A TEMP
	BSR OUTA
	LDA A 1,X
	BSR OUTA
	LDA A 2,X
;
OUTA	JMP OUTEEE
;
;  "V" COMMAND
;
VIEW	JSR BAD2
VCOM1	LDA A #8
	STA A MCONT
VCOM5	JSR CRLF
	JSR OUTXHI
	LDA B #$10
VCOM9	JSR OUT2HS
	DEC B
	BIT B #3
	BNE VCOM10
	JSR OUTS
	CMP B #$0
VCOM10	BNE VCOM9
	JSR CRLF
	LDA B #$5
	JSR SKIP
	LDX XHI
	LDA B #$10
VCOM2	LDA A 0,X
	CMP A #$20
	BCS VCOM3
	CMP A #$5F
	BCS VCOM4
VCOM3	LDA A #$2E
VCOM4	BSR OUTA
	INX
	DEC B
	BNE VCOM2
	STX XHI
	DEC MCONT
	BNE VCOM5
	JSR INEEE
	CMP A #$20
	BEQ VCOM1
	CMP A #$56
	BEQ VIEW
	JMP CONTRL
;
; 	MOTOROLA "S" LOADER PROGRAMME "S1" STARTS LOAD
; 	END OF LOAD "S9" RUN START END PLUS ADDRESS
; 	"S1" LOADER PROGRAMME START
;
SLOAD		EQU 	*
		PSHA		; Save A register
		STX	TEMPX1	; Save X register
AGAIN		JSR	GETCHARNE	; Get first CHARACTER from ACIA
		CMPA	#$53	; Is it "S"
		BNE	AGAIN	; If not go read again
		BSR	GETCHARNE	; Get second CHARACTER in frame
		CMPA 	#$39	; Is it "9"
		BEQ	RECOVER	; If "9" go and end read
		CMPA 	#$31	; Is it a "1"
		BNE	AGAIN	; If no then go start again
		CLR	TEMPA	; Clear Frame length
		BSR	GETHEX	; Get frame length from input stream
		SUBA	#$02	; Subtract the checksum
		STAA	BYTESTORE ; Save frame length
		BSR	GETADD	; Read next two bytes for dest address
GETCOUNT	BSR	GETHEX	; Get the byte number
		DEC	BYTESTORE	; decrement counter
		BEQ	INCOUNT	; If zero go to increment byte count
		STAA	0,X	; Store read byte into memory
		CMPA	0,X	; Test if RAM OK
		BNE	QUESTION ; If write failed send Question and abort
		INX		 ; Increment address pointer
		BRA	GETCOUNT ; go get another byte
;
INCOUNT		INC	TEMPA	; Increment tempa 
		BEQ	AGAIN	; If zero go for another frame
QUESTION	LDAA	#$3F	; Load question mark
		JSR	OUTPUTA	; Send to console
RECOVER		LDX	TEMPX1	; Restore "X"
		PULA		; Restore A
		JMP	CONTRL	; Jump to exit
; 
GETADD		BSR	GETHEX		; Read in byte
		STAA	ADDRESS	; store in first part of address
		BSR	GETHEX	; Get another byte of data
		STAA	ADDRESS1; store in second address register
		LDX	ADDRESS	; Load X register both bytes of address
		RTS		; Return from sub routine
;
GETHEX		BSR	CONVHEX	; Go get byte of data and convert to binary 
		ASLA		; Shift the 4 bits into msb
		ASLA		; Shift the 4 bits into msb	
		ASLA		; Shift the 4 bits into msb	
		ASLA		; Shift the 4 bits into msb	
		TAB		; Transfer "A" to "B"
		BSR	CONVHEX	; Get byte of data and convert to binary
		ABA		; Add 4 bits in "A" and "B" into "B"
		TAB		; Transfer "A" to "B"
		ADDB	TEMPA	; Add into checksum
		STAB	TEMPA	; Add into checksum
		RTS		; Return from sub routine
; 	
CONVHEX		JSR	GETCHARNE	; Get HEX CHARACTER from ACIA
		SUBA	#$30	; Convert to binary
		BMI	QUESTION ; Convert to binary
		CMPA	#$09	; Convert to binary
		BLE	RETURN2	; Convert to binary
		CMPA	#$11	; Convert to binary
		BMI	INCSTACK ; Convert to binary
		CMPA	#$16	; Convert to binary
		BGT	INCSTACK ; Convert to binary
		SUBA	#$07	; Convert to binary
RETURN2		RTS		; Return from sub routine
;  
INCSTACK	INS		; Restore stack position
		INS		; Restore stack position
		BRA	QUESTION ; Go send ? and exit
;
GETCHAR		PSHB
WAITIN		LDAB ACIACS	; LOAD ACIA CONTROL REGISTER
		ASRB		; SHIFT RIGHT  ACIADA
		BCC 	WAITIN	; IF CARRY NOT SET theN AGAIN
		LDAA 	ACIADA	; LOAD DATA REGISTER
		PULB		; RESTORE B REGISTER
		BSR 	OUTPUTA	; ECHO INPUT
		RTS

GETCHARNE	PSHB
WAITIN2		LDAB ACIACS	; LOAD ACIA CONTROL REGISTER
		ASRB		; SHIFT RIGHT  ACIADA
		BCC 	WAITIN2	; IF CARRY NOT SET theN AGAIN
		LDAA 	ACIADA	; LOAD DATA REGISTER
		PULB		; RESTORE B REGISTER
		RTS
;
OUTPUTA		PSHB		; SAVE B
WAITOUT		LDAB ACIACS	; LOAD ACIA CONTROL REGISTER
		ASRB		; SHIFT RIGHT
		ASRB		; SHIFT RIGHT
		BCC 	WAITOUT	; IF CARRY NOT SET DO AGAIN
		STAA ACIADA	; SEND CHARACTER TO ACIA
		PULB		; RESTORE B
		RTS		; RETURN FROM ROUTINE
;
OUTSTR		LDAA	0,X	; Read String
		CMPA	#$4	; Is it EOT?
		BEQ	STEXIT	; Exit if EOT
		BSR	OUTPUTA	; Print CHARACTER
		INX		; Point at next CHARACTER
		BRA	OUTSTR	; Loop and read next
STEXIT		RTS		; EXIT S1
;
; P COMMAND SENDS A BLOCK OF MEMORY IN SRECORD FORMAT

MTAPE1  FCB	$0D,$0A,'S','1',4 ; PUNCH FORMAT
MTEND	FCB	$0D,$0A,'S','9',$0D,$0A,4 ; PUNCH S9 END

PUNCH	JSR LIMITS	; GET START & END MEMORY ADDRESS
        LDX    BEGA
        STX    TW       ; TEMP BEGINNING ADDRESS
PUN11   LDA A  ENDA+1
        SUB A  TW+1
        LDA B  ENDA
        SBC B  TW
        BNE    PUN22
        CMP A  #16
        BCS    PUN23
PUN22   LDA A  #15
PUN23   ADD A  #4
        STA A  MCONT     ; FRAME COUNT THIS RECORD
        SUB A  #3
        STA A  TEMP      ; BYTE COUNT THIS RECORD

; PUNCH C/R,L/F,NULL,S,1
        LDX    #MTAPE1
        JSR    PDATA1
        CLR B            ; ZERO CHECKSUM

; PUNCH BYTE COUNT
        LDX    #MCONT
        BSR    PUNT2     ; PUNCH 2 HEX CHAR

; PUNCH ADDRESS
        LDX    #TW
        BSR    PUNT2
        BSR    PUNT2

; PUNCH DATA
        LDX    TW
PUN32   BSR    PUNT2     ; PUNCH ONE BYTE (2 FRAMES)
        DEC    TEMP      ; DEC BYTE COUNT
        BNE    PUN32
        STX    TW
        COM B
        PSH B
        TSX
        BSR    PUNT2     ; PUNCH CHECKSUM
        PUL B            ; RESTORE STACK
        LDX    TW
        DEX
        CPX    ENDA
        BNE    PUN11
; PUNCH C/R,L/F,S9,C/R,L/F
        LDX    #MTEND
        JSR    PDATA1
        JMP CONTRL	; FINISHED

; PUNCH 2 HEX CHAR UPDATE CHECKSUM
PUNT2   ADD B  0,X       ; UPDATE CHECKSUM
        JMP    OUT2H     ; OUTPUT TWO HEX CHAR AND RTS

; 16-bit addition routine, sum, diff relative jump offset - idea from NAS-SYS z80
ARITH	JSR 	BADDR		; get 1st arg
	STX 	BEGA		; store
	JSR 	OUTS		; space
	JSR 	BADDR		; get 2nd arg
	STX 	ENDA		; store
	LDX 	#ARITHM		; 'sum diff rel' header
	JSR 	PDATA1
	CLC
	LDAA	BEGA+1
	ADC A	ENDA+1
	STAA	LENGTH2
	LDAA	BEGA
	ADC A	ENDA
	STAA	LENGTH
	LDX	#LENGTH
	JSR	OUT4HS

; 16-bit subtract
        
	CLC         		; clear the carry
	LDAA	ENDA+1		; get the low byte of the first number
	SBC A	BEGA+1		; subr the low byte of the second
	STAA	LENGTH+1	; store in the low byte of the result
	LDAA	ENDA		; get the high byte of the first number
	SBC A	BEGA		; subtr the high byte of the second, plus carry
	STAA	LENGTH		; store in high byte of the result
	LDX	#LENGTH
	JSR	OUT4HS

; relative jump offset

	LDX	LENGTH   	; DEC HL, DEC HL
	DEX
	DEX
	STX	LENGTH		; SAVE
	LDAA	LENGTH  	; LD A,H
	ROL	LENGTH+1   	; RLC L
	ADC A	#0		; ADC A,0
	BEQ	RJOK		; ZERO IF REL JMP IS POSSIBLE
	LDX 	#DQUES
	JSR 	PDATA1
	BRA 	ADCR
RJOK	ROR	LENGTH2
	LDA A	LENGTH2	; LD A,L
	LDX	#LENGTH2
	JSR	OUT2HS  
ADCR	JSR	CRLF      
ADONE	JMP	CONTRL	; monitor 	
;
HELP	LDX #FRONT
	JSR PDATA1
	BRA ADONE
;
; MNEMONIC TABLE
;
TBL	FCC "***NOPNOP***"
	FCC "******TAPTPA"
	FCC "INXDEXCLVSEV"
	FCC "CLCSECCLISEI"
	FCC "SBACBA******"
	FCC "******TABTBA"
	FCC "***DAA***ABA"
	FCC "************"
	FCC "BRA***BHIBLS"
	FCC "BCCBCSBNEBEQ"
	FCC "BVCBVSBPLBMI"
	FCC "BGEBLTBGTBLE"
	FCC "TSXINSPULPUL"
	FCC "DESTXSPSHPSH"
	FCC "***RTS***RTI"
	FCC "******WAISWI"
	FCC "NEG******COM"
	FCC "LSR***RORASR"
	FCC "ASLROLDEC***"
	FCC "INCTSTJMPCLR"
	FCC "SUBCMPSBCBSR"
	FCC "ANDBITLDASTA"
	FCC "EORADCORAADD"
	FCC "CPXJSRLDSSTS"
	FCC "LDXSTX"
SPLBD0  FCC "#$"
	 FCB $4
;
COMMX	 FCB $2C,$58,$04
BLANK	 FCB $20,$20,$20
	 FCB $04
PNTA	 FCC " A "
	 FCB $04
PNTB	 FCC " B "
	 FCB $04
MCL	 FCB $0D,$0A,$3E,$04
BREG	 FCB $20,$42,$3D,$04
AREG	 FCB $41,$3D,$04
XREG	 FCB $58,$3D,$04
SREG	 FCB $53,$3D,$04
PCTR	 FCB $50,$43,$3D,$04
CSET	 FCB $48,$49,$4E,$5A,$56,$43
CRLFAS	 FCB $0D,$0A,$04
ADASC	 FCB $0D,$0A
	 FCB $42,$4B,$41,$44,$44,$52,$20,$04
FROMAD   FCB $0D,$0A,$46,$52,$4F,$4D,$20
	 FCB $41,$44,$44,$52,$20,$04
THRUAD   FCB $0D,$0A,$54,$48,$52,$55,$20,$41
	 FCB $44,$44,$52,$20,$04
TOADD	 FCB $54,$4F,$20,$41,$44,$44,$52,$20,$04
VALASC   FCB $56,$41,$4C,$55,$45,$20,$04
ARITHM	 FCB $0D,$0A
	 FCC "SUM  DIFF RELJMP"
	 FCB $0D,$0A,$04
DQUES	 FCC "?? "
	 FCB $0D,$0A,$04

;
;	FRONT SCREEN
;
FRONT	FCB	$0D, $0A
	FCC     "               INSTRUCTION SET "
	FCB	$0A, $0D
	FCC     "               =============== "
	FCB	$0A, $0D
	FCC	" A = SET A REGISTER          N = ECHO OFF "
	FCB	$0A, $0D
	FCC	" B = SET B REGISTER          O = ECHO ON "
	FCB	$0A, $0D
	FCC	" C = SET PROGRAM COUNTER     P = PUNCH S-RECORDS "
	FCB	$0A, $0D
	FCC	" D = DISASSEMBLE CODE        Q = HARDWARE LOCATION "
	FCB	$0A, $0D
	FCC	" E = CHANGE MEMORY           R = PRINT REGISTERS "
	FCB	$0A, $0D
	FCC	" F = FIND                    S = SET STACK POINTER "
	FCB	$0A, $0D
	FCC	" G = GO TO PROGRAM           T = TRACE PROGRAM "
	FCB	$0A, $0D
	FCC	" H = HELP COMMAND LIST       U = TSC MICROBASIC+"
	FCB	$0A, $0D
	FCC	" I = FILL MEMORY             V = VIEW MEMORY "
	FCB	$0A, $0D
	FCC	" J = JUMP TO TARGET PROGRAM  X = SET X REGISTER "
	FCB	$0A, $0D
	FCC	" K = CONTINUE AFTER BREAK    @ = ASCII CONVERSION "
	FCB	$0A, $0D
	FCC	" L = LOAD S-RECORDS          1 = BREAKPOINT ONE "
	FCB	$0A, $0D
	FCC	" M = MOVE MEMORY             2 = BREAKPOINT TWO "
	FCB	$0A, $0D
	FCC	"                             + = HEX ARITHMETIC "
	FCB	$0D, $0A,$04
MONPR	FCB	$0D, $0A
	FCC	" CMD (H=HELP) >"
	FCB	$04
;
;   COMMAND JUMP TABLE
;
FUTABL		FCC "M"
		FDB MOVE
		FCC "E"
		FDB CHANGE
		FCC "G"
		FDB CONTG
		FCC "R"
		FDB PRINT
		FCC "T"
		FDB TRACE
		FCC "@"
		FDB ASCII
		FCC "H"
		FDB HELP
		FCC "V"
		FDB VIEW
		FCC "I"
		FDB IFILL
		FCC "J"
		FDB JUMP
		FCC "F"
		FDB FIND
		FCC "Q"
		FDB START
		FCC "D"
		FDB DISSA
		FCC "K"
		FDB KONTIN
		FCC "1"
		FDB BKPNT
		FCC "2"
		FDB BKPNT2
		FCC "L"
		FDB SLOAD
		FCC "O"
		FDB ECHON
		FCC "N"
		FDB ECHOFF
		FCC "P"
		FDB PUNCH
		FCC "U"
		FDB MICROB
		FCC "+"
		FDB ARITH
TBLEND	EQU *
;
MICROB	LDX #MBCODE		; FROM
	STX BEGA
	LDX #MBCODE2		; THROUGH
	STX ENDA
	LDX #$0100		; TO
	STX XHI
	JSR MOVE2
	JMP $0100  	; TSC MICROBASIC PLUS

	ORG $F300

MBCODE	EQU *
	FCB $7E,$01,$F3,$7E,$02,$03,$7E,$E1,$D1,$BD,$E1,$AC,$7E,$04,$A4,$1E,$FF,$50,$52,$49,$04,$FA,$49,$4E,$50,$07,$E9,$49,$46,$20,$09,$03
	FCB $4C,$45,$54,$07,$C5,$46,$4F,$52,$09,$C7,$4E,$45,$58,$09,$EE,$47,$4F,$54,$07,$D3,$47,$4F,$53,$09,$7C,$4F,$4E,$20,$08,$C7,$52,$45
	FCB $54,$09,$A4,$52,$45,$41,$08,$77,$44,$41,$54,$08,$68,$52,$45,$53,$08,$BD,$44,$49,$4D,$06,$C4,$45,$58,$54,$07,$54,$4D,$4F,$4E,$E0
	FCB $E3,$45,$4E,$44,$02,$03,$52,$45,$4D,$07,$57,$52,$55,$4E,$07,$B2,$4C,$49,$53,$04,$3F,$53,$43,$52,$01,$F3,$00,$52,$4E,$44,$0B,$11
	FCB $41,$42,$53,$0B,$0D,$53,$47,$4E,$0B,$05,$00,$CE,$01,$00,$DF,$0A,$CE,$00,$0C,$20,$08,$FE,$01,$0F,$DF,$0A,$FE,$0D,$9D,$4F,$A7,$00
	FCB $08,$9C,$0A,$26,$F9,$39,$0D,$0A,$0A,$4D,$49,$43,$52,$4F,$20,$42,$41,$53,$49,$43,$20,$50,$4C,$55,$53,$0D,$0A,$43,$4F,$50,$59,$52
	FCB $49,$47,$48,$54,$20,$28,$43,$29,$20,$31,$39,$37,$36,$20,$42,$59,$0D,$0A,$54,$45,$43,$48,$4E,$49,$43,$41,$4C,$20,$53,$59,$53,$54
	FCB $45,$4D,$53,$20,$43,$4F,$4E,$53,$55,$4C,$54,$41,$4E,$54,$53,$0D,$0A,$0A,$04,$CE,$01,$A6,$BD,$03,$43,$8D,$90,$CE,$10,$00,$FF,$0D
	FCB $9D,$8D,$92,$CE,$01,$03,$FF,$A0,$48,$8E,$A0,$7F,$CE,$00,$68,$DF,$0A,$BD,$01,$90,$CE,$0D,$9D,$FF,$A0,$02,$EE,$00,$FF,$A0,$04,$DF
	FCB $08,$CE,$00,$68,$BD,$03,$3E,$86,$21,$BD,$04,$9E,$BD,$03,$24,$27,$D2,$A7,$00,$81,$0D,$27,$08,$08,$8C,$00,$B0,$26,$EF,$20,$C4,$CE
	FCB $00,$68,$BD,$03,$85,$DF,$31,$BD,$03,$CF,$4D,$26,$1A,$DE,$04,$A6,$00,$81,$0D,$26,$08,$D6,$28,$27,$AA,$97,$12,$20,$0A,$BD,$07,$98
	FCB $27,$05,$86,$10,$7E,$04,$B7,$96,$3E,$90,$26,$97,$3E,$D6,$28,$26,$06,$BD,$03,$3E,$7E,$07,$94,$FE,$01,$0F,$DF,$37,$DE,$31,$DF,$04
	FCB $BD,$02,$F9,$DF,$22,$5D,$26,$20,$5C,$A6,$00,$08,$81,$0D,$26,$F8,$F7,$02,$A0,$86,$FF,$50,$8D,$46,$DE,$22,$BC,$0D,$9D,$27,$07,$A6
	FCB $00,$A7,$00,$08,$20,$F4,$DE,$22,$96,$12,$26,$2F,$FE,$0D,$9D,$D6,$3E,$CB,$02,$F7,$02,$C0,$8D,$26,$9C,$22,$27,$07,$09,$A6,$00,$A7
	FCB $00,$20,$F5,$09,$BD,$06,$BB,$08,$08,$DF,$22,$DE,$04,$A6,$00,$08,$DF,$04,$DE,$22,$08,$A7,$00,$81,$0D,$26,$EE,$7E,$02,$03,$FB,$0D
	FCB $9E,$B9,$0D,$9D,$D7,$3A,$97,$39,$BD,$0D,$03,$24,$07,$F7,$0D,$9E,$B7,$0D,$9D,$39,$86,$90,$7E,$04,$B7,$96,$64,$D6,$63,$CE,$10,$00
	FCB $BC,$0D,$9D,$26,$02,$5C,$39,$E1,$00,$22,$0A,$26,$F8,$A1,$01,$22,$04,$26,$F2,$5F,$39,$8D,$03,$08,$20,$E6,$36,$86,$0D,$08,$A1,$00
	FCB $26,$FB,$32,$39,$BD,$01,$09,$81,$08,$26,$0B,$8C,$00,$68,$27,$0D,$09,$7A,$00,$3E,$20,$EE,$81,$18,$27,$03,$7C,$00,$3E,$39,$DF,$22
	FCB $CE,$03,$55,$A6,$00,$81,$04,$27,$06,$BD,$04,$9E,$08,$20,$F4,$DE,$22,$7F,$00,$1D,$39,$0D,$0A,$00,$00,$00,$00,$04,$81,$0D,$27,$02
	FCB $81,$3A,$39,$BD,$0B,$A2,$4F,$97,$62,$97,$63,$97,$64,$39,$8D,$F6,$97,$28,$97,$27,$97,$26,$BD,$03,$BC,$81,$2B,$27,$07,$81,$2D,$26
	FCB $04,$73,$00,$27,$08,$BD,$0D,$33,$C1,$03,$27,$05,$96,$27,$7E,$0C,$3B,$08,$97,$28,$84,$0F,$C6,$04,$78,$00,$64,$79,$00,$63,$79,$00
	FCB $62,$5A,$26,$F4,$9B,$64,$97,$64,$7C,$00,$26,$20,$D8,$DE,$04,$A6,$00,$81,$20,$27,$07,$08,$20,$F7,$8D,$B4,$09,$08,$A6,$00,$81,$20
	FCB $27,$F9,$39,$DE,$04,$BD,$0D,$33,$C1,$02,$26,$F0,$08,$20,$F6,$8D,$EB,$DF,$04,$DF,$22,$CE,$01,$11,$C6,$05,$A1,$00,$26,$12,$DF,$0A
	FCB $DE,$22,$08,$A6,$00,$DF,$22,$DE,$0A,$08,$5A,$C1,$02,$26,$EB,$39,$08,$5A,$26,$FC,$A6,$00,$27,$F7,$DF,$0A,$DE,$04,$DF,$22,$A6,$00
	FCB $DE,$0A,$20,$D4,$CE,$00,$62,$C6,$02,$0C,$A6,$00,$2A,$19,$86,$2D,$BD,$04,$9E,$7C,$00,$1D,$20,$0F,$A6,$00,$85,$F0,$25,$02,$27,$07
	FCB $BD,$04,$96,$7C,$00,$1D,$0D,$A6,$00,$C5,$FF,$27,$06,$85,$0F,$25,$02,$27,$07,$BD,$04,$9A,$7C,$00,$1D,$0D,$08,$5A,$2A,$DA,$39,$8D
	FCB $82,$81,$0D,$27,$25,$BD,$03,$6E,$DF,$04,$BD,$02,$F9,$DF,$22,$BD,$03,$C3,$81,$0D,$26,$05,$7C,$00,$1B,$20,$0B,$08,$BD,$03,$BC,$BD
	FCB $03,$6E,$96,$64,$97,$1B,$DE,$22,$20,$03,$CE,$10,$00,$BC,$0D,$9D,$27,$21,$BD,$03,$3E,$C6,$01,$0C,$8D,$9E,$A6,$00,$81,$0D,$27,$05
	FCB $8D,$1C,$08,$20,$F5,$08,$96,$1B,$27,$E3,$8B,$99,$19,$27,$04,$97,$1B,$20,$DA,$7E,$02,$03,$44,$44,$44,$44,$84,$0F,$8B,$30,$BD,$01
	FCB $0C,$7E,$01,$06,$36,$B6,$80,$04,$84,$01,$26,$02,$32,$39,$B6,$80,$05,$81,$03,$26,$F7,$86,$99,$36,$BD,$03,$3E,$CE,$04,$EC,$BD,$03
	FCB $43,$32,$36,$8D,$D1,$32,$8D,$D2,$D6,$19,$26,$03,$7E,$02,$03,$CE,$04,$F5,$BD,$03,$43,$DE,$04,$09,$8C,$10,$00,$27,$07,$A6,$00,$81
	FCB $0D,$26,$F4,$08,$C6,$01,$0C,$BD,$04,$18,$20,$E0,$07,$45,$52,$52,$4F,$52,$20,$23,$04,$20,$41,$54,$20,$04,$BD,$03,$C3,$BD,$03,$5C
	FCB $26,$03,$7E,$05,$8F,$7F,$00,$12,$81,$2C,$26,$1F,$D6,$1D,$86,$20,$8D,$8C,$5C,$C5,$07,$26,$F7,$C1,$47,$22,$04,$D7,$1D,$20,$03,$BD
	FCB $03,$3E,$7C,$00,$12,$08,$BD,$03,$BC,$20,$D2,$81,$3B,$27,$F3,$81,$22,$26,$05,$08,$8D,$64,$20,$49,$7F,$00,$17,$81,$54,$26,$06,$97
	FCB $17,$86,$41,$20,$06,$81,$53,$26,$2E,$86,$50,$A1,$01,$26,$28,$BD,$03,$C5,$BD,$0A,$77,$BD,$06,$71,$D6,$64,$27,$25,$96,$17,$27,$07
	FCB $5A,$D1,$1D,$23,$1C,$20,$02,$DB,$1D,$86,$20,$BD,$04,$9E,$7C,$00,$1D,$D1,$1D,$26,$F4,$20,$0A,$BD,$0A,$77,$DF,$22,$BD,$04,$04,$DE
	FCB $22,$BD,$0D,$2E,$5A,$26,$03,$7E,$04,$FD,$86,$31,$7E,$04,$B7,$7D,$00,$12,$26,$03,$BD,$03,$3E,$7E,$07,$57,$A6,$00,$81,$22,$27,$0E
	FCB $BD,$03,$5C,$27,$0D,$BD,$04,$9E,$7C,$00,$1D,$08,$20,$EC,$08,$7E,$03,$BC,$86,$32,$7E,$04,$B7,$DF,$04,$BD,$0D,$35,$C1,$02,$26,$2F
	FCB $7F,$00,$20,$16,$48,$1B,$80,$13,$97,$21,$DE,$20,$39,$A6,$00,$08,$7F,$00,$18,$8D,$E2,$5F,$A6,$00,$81,$0A,$27,$06,$81,$0B,$27,$01
	FCB $39,$5C,$A6,$01,$36,$A6,$02,$36,$37,$BD,$03,$C3,$33,$81,$28,$26,$71,$5D,$27,$13,$08,$BD,$0A,$7A,$96,$64,$36,$BD,$0B,$B3,$BD,$03
	FCB $C3,$81,$2C,$26,$5D,$20,$02,$4F,$36,$4C,$97,$18,$08,$BD,$0A,$7A,$08,$DF,$04,$32,$97,$14,$32,$97,$21,$32,$97,$20,$DE,$20,$A6,$00
	FCB $97,$16,$08,$08,$DF,$20,$BD,$03,$63,$96,$14,$DE,$20,$09,$A1,$00,$22,$30,$97,$64,$BD,$03,$63,$96,$16,$91,$5E,$27,$02,$23,$23,$8B
	FCB $01,$19,$97,$64,$BD,$0C,$45,$BD,$0C,$1B,$BD,$06,$67,$96,$20,$D6,$21,$DB,$64,$99,$63,$97,$20,$D7,$21,$BD,$0B,$B3,$DE,$20,$7F,$00
	FCB $18,$39,$86,$14,$7E,$04,$B7,$BD,$03,$63,$86,$03,$97,$64,$BD,$0C,$45,$96,$64,$36,$96,$63,$36,$5F,$D7,$63,$D7,$64,$96,$62,$8D,$12
	FCB $32,$36,$8D,$0A,$32,$8D,$0B,$32,$36,$8D,$03,$32,$20,$1D,$44,$44,$44,$44,$8D,$17,$D6,$63,$48,$59,$37,$36,$48,$59,$48,$59,$97,$64
	FCB $32,$D7,$63,$8D,$08,$32,$9B,$63,$97,$63,$39,$84,$0F,$9B,$64,$97,$64,$24,$03,$7C,$00,$63,$39,$96,$62,$A7,$00,$96,$63,$A7,$01,$96
	FCB $64,$A7,$02,$39,$DE,$06,$DF,$37,$BD,$03,$C3,$BD,$03,$BC,$BD,$05,$B7,$DF,$0A,$BD,$03,$C3,$81,$28,$26,$20,$08,$BD,$03,$B8,$81,$29
	FCB $26,$05,$4F,$5F,$36,$20,$18,$81,$2C,$26,$0F,$96,$64,$27,$0B,$36,$08,$BD,$03,$B8,$C6,$01,$81,$29,$27,$05,$86,$40,$7E,$04,$B7,$96
	FCB $64,$27,$F7,$36,$DF,$04,$DE,$0A,$86,$0A,$1B,$A7,$00,$96,$08,$A7,$01,$96,$09,$A7,$02,$DE,$08,$32,$A7,$00,$08,$33,$E7,$00,$08,$DF
	FCB $20,$8B,$01,$19,$36,$17,$8B,$01,$19,$16,$BD,$03,$66,$D7,$64,$BD,$03,$63,$32,$97,$64,$BD,$0C,$45,$BD,$06,$4A,$BD,$0D,$01,$23,$03
	FCB $7E,$02,$F4,$DF,$08,$DE,$04,$08,$BD,$03,$BC,$BD,$03,$5C,$27,$07,$08,$7E,$06,$CB,$BD,$1F,$00,$4F,$97,$12,$97,$1C,$97,$18,$97,$2C
	FCB $96,$19,$26,$03,$7E,$02,$03,$DE,$04,$86,$0D,$C6,$3A,$A1,$00,$27,$07,$E1,$00,$27,$0A,$08,$20,$F5,$08,$BC,$0D,$9D,$27,$E6,$08,$08
	FCB $BD,$01,$0C,$BD,$03,$CF,$4D,$26,$0B,$DE,$04,$8D,$0B,$27,$05,$86,$10,$7E,$04,$B7,$EE,$00,$6E,$00,$BD,$0D,$33,$C1,$02,$26,$12,$08
	FCB $BD,$03,$BC,$81,$3D,$27,$04,$81,$28,$26,$06,$CE,$01,$23,$97,$1C,$5F,$39,$BD,$01,$8B,$BD,$01,$95,$FE,$01,$0F,$DF,$06,$CE,$10,$00
	FCB $7C,$00,$19,$20,$B4,$DE,$04,$96,$1C,$26,$03,$BD,$03,$AD,$BD,$09,$B6,$20,$84,$BD,$03,$C3,$BD,$0A,$77,$BD,$02,$F9,$5D,$27,$05,$86
	FCB $16,$7E,$04,$B7,$5C,$D7,$19,$20,$90,$BD,$03,$C3,$7F,$00,$13,$BD,$03,$BC,$81,$22,$26,$06,$08,$BD,$05,$9A,$20,$3B,$BD,$05,$CF,$DF
	FCB $33,$CE,$00,$68,$96,$13,$26,$07,$86,$3F,$97,$13,$BD,$04,$9E,$BD,$01,$09,$81,$18,$26,$05,$7F,$00,$13,$20,$E6,$A7,$00,$08,$81,$2C
	FCB $27,$09,$81,$0D,$26,$E9,$97,$12,$BD,$03,$3E,$CE,$00,$68,$BD,$03,$6E,$DE,$33,$8D,$2D,$DF,$04,$81,$2C,$26,$07,$08,$96,$12,$27,$AF
	FCB $20,$AA,$BD,$03,$5C,$26,$13,$96,$12,$27,$03,$7E,$07,$57,$BD,$01,$09,$81,$0D,$26,$F9,$BD,$03,$3E,$20,$F1,$86,$45,$7E,$04,$B7,$BD
	FCB $05,$CF,$BD,$06,$B7,$7E,$03,$C3,$96,$19,$27,$49,$BD,$03,$C3,$97,$1A,$DF,$0C,$DF,$0E,$20,$3E,$96,$19,$27,$3A,$96,$1A,$27,$39,$BD
	FCB $03,$AD,$BD,$03,$BC,$BD,$05,$CF,$DF,$33,$DE,$04,$DF,$35,$DE,$0E,$BD,$0A,$77,$A6,$00,$BD,$03,$5C,$26,$04,$DE,$0C,$20,$01,$08,$DF
	FCB $0E,$DE,$35,$DF,$04,$DE,$33,$8D,$B9,$81,$2C,$26,$03,$08,$20,$D2,$BD,$03,$5C,$26,$03,$7E,$07,$57,$86,$51,$7E,$04,$B7,$DF,$22,$DE
	FCB $0C,$DF,$0E,$DE,$22,$20,$EE,$BD,$03,$AD,$BD,$0A,$77,$96,$64,$84,$0F,$36,$7F,$00,$12,$08,$08,$A6,$00,$81,$54,$27,$02,$97,$12,$BD
	FCB $03,$AF,$DF,$22,$32,$4A,$27,$11,$E6,$00,$08,$C1,$2C,$26,$04,$DF,$22,$20,$F2,$C1,$0D,$26,$F1,$DE,$22,$D6,$12,$27,$03,$7E,$09,$83
	FCB $7E,$07,$D6,$BD,$03,$C3,$BD,$0A,$77,$A6,$00,$8D,$63,$26,$5C,$36,$A6,$01,$8D,$5C,$32,$26,$04,$E6,$01,$1B,$08,$08,$36,$BD,$0A,$77
	FCB $32,$84,$0F,$80,$09,$2B,$44,$48,$48,$B7,$09,$33,$BD,$0C,$15,$BD,$0D,$0E,$20,$FE,$2F,$18,$20,$30,$26,$14,$20,$2C,$2C,$10,$20,$28
	FCB $2D,$0C,$20,$24,$27,$08,$20,$20,$2E,$04,$20,$1C,$20,$1D,$DE,$04,$A6,$00,$81,$54,$26,$0F,$BD,$03,$C3,$DF,$04,$BD,$0D,$35,$C1,$03
	FCB $26,$03,$7E,$07,$D6,$7E,$07,$83,$7E,$07,$57,$86,$62,$7E,$04,$B7,$81,$3B,$23,$06,$81,$3E,$22,$02,$5F,$39,$5C,$39,$D6,$19,$27,$E8
	FCB $BD,$03,$C3,$7C,$00,$1B,$BD,$0A,$77,$09,$BD,$03,$1A,$08,$A6,$00,$36,$A6,$01,$36,$9F,$37,$CE,$A0,$23,$BD,$0D,$01,$23,$03,$7E,$02
	FCB $F4,$7E,$07,$D9,$86,$73,$7A,$00,$1B,$2A,$03,$7E,$04,$B7,$32,$33,$BD,$02,$FD,$7E,$07,$DC,$BD,$05,$CD,$DF,$33,$BD,$03,$C3,$08,$BD
	FCB $0A,$77,$DE,$33,$7E,$06,$B7,$BD,$03,$AD,$36,$8D,$E9,$DE,$08,$DF,$37,$DE,$06,$32,$A7,$00,$96,$05,$09,$A7,$00,$96,$04,$09,$A7,$00
	FCB $09,$BD,$0D,$01,$22,$03,$7E,$02,$F4,$DF,$06,$7E,$07,$57,$BD,$03,$AD,$DF,$1E,$DE,$06,$BC,$01,$0F,$26,$04,$DE,$04,$20,$74,$08,$08
	FCB $08,$A1,$00,$26,$F0,$09,$09,$09,$DF,$06,$08,$EE,$00,$DF,$04,$BD,$05,$CF,$DF,$33,$BD,$03,$C3,$BD,$0A,$77,$BD,$0B,$A2,$DE,$33,$BD
	FCB $0B,$95,$DE,$04,$A6,$00,$81,$53,$27,$08,$BD,$03,$63,$4C,$97,$64,$20,$0A,$BD,$03,$C5,$BD,$0A,$77,$96,$62,$97,$1C,$BD,$0C,$1B,$CE
	FCB $00,$10,$BD,$06,$B7,$BD,$0C,$15,$BD,$0D,$0E,$D6,$1C,$2B,$05,$06,$2C,$12,$20,$03,$06,$2F,$0D,$DE,$06,$08,$08,$08,$DF,$06,$DE,$1E
	FCB $DF,$04,$20,$0B,$CE,$00,$10,$BD,$0B,$95,$DE,$33,$BD,$06,$B7,$7E,$07,$57,$86,$81,$7E,$04,$B7,$7F,$00,$2C,$96,$2C,$97,$2D,$8D,$04
	FCB $4D,$26,$F1,$39,$9F,$FE,$BD,$0D,$2E,$DF,$04,$C1,$01,$26,$04,$86,$21,$20,$4A,$54,$C1,$03,$26,$03,$BD,$03,$63,$DE,$04,$BD,$0D,$2E
	FCB $DF,$04,$C1,$04,$23,$02,$C6,$05,$58,$F7,$0A,$AD,$20,$FE,$20,$2B,$20,$1B,$20,$38,$20,$2C,$20,$04,$36,$08,$20,$CA,$30,$09,$D6,$18
	FCB $9C,$FE,$27,$06,$32,$5F,$81,$28,$27,$F2,$5D,$27,$0E,$4F,$D6,$2C,$5A,$D1,$2D,$26,$06,$30,$09,$9C,$FE,$27,$04,$86,$20,$9E,$FE,$DE
	FCB $04,$39,$BD,$0B,$A2,$DE,$04,$BD,$03,$6E,$20,$59,$A6,$01,$BD,$0D,$35,$C1,$02,$26,$28,$A6,$00,$DF,$22,$CE,$01,$7B,$BD,$03,$D8,$4D
	FCB $27,$CB,$7E,$07,$94,$86,$3F,$36,$DE,$22,$7E,$0A,$86,$86,$40,$20,$F6,$BD,$03,$63,$BD,$0D,$7A,$97,$64,$DE,$22,$20,$28,$D6,$FE,$37
	FCB $D6,$FF,$37,$D6,$2D,$37,$D6,$18,$37,$BD,$05,$CD,$33,$D7,$18,$33,$D7,$2D,$33,$D7,$FF,$33,$D7,$FE,$BD,$0B,$A2,$DE,$20,$BD,$0B,$95
	FCB $20,$05,$DE,$04,$08,$DF,$04,$30,$09,$9C,$FE,$27,$37,$32,$36,$81,$28,$27,$31,$BD,$0D,$35,$37,$54,$96,$2C,$4A,$C1,$04,$27,$04,$91
	FCB $2D,$27,$21,$81,$09,$23,$04,$86,$24,$20,$16,$32,$33,$80,$06,$48,$B7,$0B,$77,$CE,$0B,$87,$EE,$00,$AD,$00,$BD,$0D,$0E,$28,$C8,$86
	FCB $23,$7E,$0A,$DD,$7E,$0A,$9B,$0C,$1B,$0C,$15,$0C,$D2,$0C,$0D,$0C,$45,$0C,$66,$0C,$E4,$A6,$00,$97,$62,$A6,$01,$97,$63,$A6,$02,$97
	FCB $64,$39,$CE,$00,$3B,$E6,$03,$E7,$00,$08,$8C,$00,$62,$26,$F6,$7C,$00,$2C,$39,$CE,$00,$64,$E6,$00,$E7,$03,$09,$8C,$00,$3A,$26,$F6
	FCB $7A,$00,$2C,$39,$0C,$CE,$00,$64,$A6,$00,$A9,$03,$19,$A7,$00,$09,$8C,$00,$61,$26,$F3,$37,$C6,$02,$85,$F0,$26,$01,$5F,$DA,$30,$D7
	FCB $30,$17,$33,$39,$8D,$03,$0D,$20,$DC,$CE,$00,$67,$86,$99,$A0,$00,$A7,$00,$09,$8C,$00,$64,$26,$F4,$84,$0F,$A7,$01,$39,$7F,$00,$30
	FCB $96,$65,$16,$C4,$0F,$D7,$65,$97,$2F,$98,$62,$97,$2E,$D6,$62,$C4,$0F,$D7,$62,$4D,$39,$96,$62,$88,$F0,$97,$62,$8D,$58,$8D,$DE,$2A
	FCB $0A,$8D,$C1,$06,$28,$09,$73,$00,$2F,$20,$0B,$8D,$97,$20,$0A,$8D,$44,$BD,$03,$63,$8D,$AE,$7F,$00,$30,$96,$2F,$84,$F0,$C6,$0F,$D4
	FCB $62,$1B,$97,$62,$39,$8D,$2E,$8D,$B4,$BD,$03,$63,$C6,$05,$96,$5F,$27,$08,$BD,$0B,$C4,$7A,$00,$5F,$26,$F8,$5A,$27,$3C,$8D,$49,$96
	FCB $62,$BD,$0B,$D5,$20,$E8,$8D,$0D,$CE,$00,$65,$BD,$0D,$11,$26,$08,$86,$22,$7E,$0A,$DD,$7E,$0B,$B3,$8D,$83,$BD,$0B,$A2,$8D,$29,$6F
	FCB $02,$6F,$03,$C6,$05,$8D,$21,$BD,$0B,$E9,$8D,$2E,$85,$F0,$26,$13,$BD,$0B,$E9,$0C,$8D,$25,$5A,$26,$EC,$96,$2E,$8D,$9E,$CE,$00,$5E
	FCB $7E,$0B,$B6,$7C,$00,$64,$20,$E2,$86,$04,$CE,$00,$64,$0C,$69,$00,$09,$8C,$00,$5E,$26,$F8,$4A,$26,$F1,$39,$0D,$CE,$00,$61,$96,$5F
	FCB $84,$0F,$97,$5F,$A6,$00,$A9,$06,$19,$A7,$00,$09,$8C,$00,$5E,$26,$F3,$39,$8D,$3A,$27,$0B,$D6,$62,$8D,$07,$7C,$00,$64,$17,$7E,$0C
	FCB $3B,$7E,$03,$66,$8D,$8F,$5F,$D7,$30,$96,$67,$27,$EB,$BD,$0B,$A2,$8D,$83,$8B,$99,$19,$27,$16,$36,$BD,$0C,$00,$BD,$0C,$49,$32,$20
	FCB $F1,$DF,$39,$96,$39,$91,$37,$26,$04,$D6,$3A,$D1,$38,$39,$CE,$00,$62,$5F,$6D,$02,$26,$0E,$6D,$01,$26,$0A,$A6,$00,$84,$0F,$26,$04
	FCB $A7,$00,$C6,$04,$A6,$00,$46,$84,$08,$1B,$9A,$30,$06,$39,$BD,$03,$BC,$20,$02,$A6,$00,$C6,$01,$81,$0D,$27,$17,$5A,$36,$80,$28,$2B
	FCB $10,$81,$18,$23,$0E,$81,$32,$23,$06,$81,$36,$26,$04,$C6,$0A,$CB,$02,$32,$39,$DF,$24,$CE,$0D,$61,$B7,$0D,$5C,$E6,$00,$DE,$24,$20
	FCB $F0,$05,$04,$0A,$06,$01,$07,$00,$0B,$03,$03,$03,$03,$03,$03,$03,$03,$03,$03,$01,$01,$01,$01,$01,$08,$09,$C6,$08,$CE,$00,$00,$A6
	FCB $03,$48,$48,$48,$A8,$03,$48,$48,$69,$00,$69,$01,$69,$02,$69,$03,$5A,$26,$EC,$A6,$00,$81,$9F,$22,$E1,$8B,$00,$19,$39,$00,$00
	FCB $39
MBCODE2	EQU *




;
;   STARTUP VECTORS $FFF8 -$FFFF
;
		ORG $FFF8
;
		FDB IO
		FDB SWI
		FDB POWDWN
		FDB START
;
            END
;
